# perform one query on the resulting table that makes use of the PostgresSQL XPath() function (I am being flexible on what you do, but it should demo you understand XPath syntax) and output the result (should be non-empty) to the terminal.


import psycopg2
import os

DB_NAME = "shopping"
DB_USER = "dhruv"
DB_PASSW = ""
DB_HOST = "127.0.0.1"
DB_PORT = "5432"


def connect_db(db_name, db_user, db_passw, db_host, db_port):
    connection = None
    try:
        connection = psycopg2.connect(
            database=db_name, host=db_host, port=db_port, user=db_user, password=db_passw)
        print("Connection successful")
    except psycopg2.OperationalError as e:
        print(f"Error '{e}' occurred while connecting")
    return connection


def execute_query(connection, query):
    connection.autocommit = True
    cursor = connection.cursor()
    try:
        cursor.execute(query)
    except psycopg2.OperationalError as e:
        print(f"Error '{e}'")


def read_query(connection, query):
    cursor = connection.cursor()
    result = None
    try:
        cursor.execute(query)
        result = cursor.fetchall()
        return result
    except psycopg2.OperationalError as e:
        print(f"Error '{e}'")


connect_obj = connect_db(DB_NAME, DB_USER, DB_PASSW, DB_HOST, DB_PORT)

create_table = """
    CREATE TABLE IF NOT EXISTS Cart (
        ID SERIAL PRIMARY KEY,
        FILE_NAME char(20),
        FILE_CONTENT XML
    )
"""

execute_query(connect_obj, create_table)

row_entries = []
for filename in os.listdir():
    if filename.startswith("cart") and filename.endswith(".xml"):
        with open(filename, 'r') as reader:
            xml_content = reader.read()
            # xml entries from all files are appended in one list to optimise SQL request
            row_entries.append(
                f"('{filename}', XMLPARSE(DOCUMENT '{xml_content}'))")


all_rows = ", ".join(row_entries)
insert_entry = f"INSERT INTO Cart (FILE_NAME, FILE_CONTENT) VALUES {all_rows}"
execute_query(connect_obj, insert_entry)


# show result after insertion
select_query = f"SELECT * FROM Cart"
read_rows = read_query(connect_obj, select_query)

for row in read_rows:
    print(row, end='\n\n')


# xpath query to find all items in all the cart rows without repeating
xpath_query = f"SELECT DISTINCT CAST(unnest(xpath('//name/text()', FILE_CONTENT)) AS CHAR(50)) FROM Cart"
read_rows = read_query(connect_obj, xpath_query)

# show result after reading
for row in read_rows:
    print(row, end='\n\n')
